/// <reference types="jest" />
import { MockFunctionReturnSpy } from "./base";
export interface Spies {
    [name: string]: jest.SpyInstance;
}
/**
 * MockChain provides chaining mocks functionalities
 * @example
    // construct mock chain
    const chain = new MockChain()
    chain
        .add(V3.mockDynamo.send, 1) <-- stack this mock behavior to spies[0]
        .add(V3.mockLambda.send, {}) <-- stack this mock behavior to spies[1]
        .add(V3.mockDynamo.send, 2, 0) <-- add this mock behavior to spies[0]
            (The Send method of DynamoDB will be called twice in the test target)
    const spies = chain.getSpies()

    // checking spies
    expect(spies[0]).toHaveBeenNthCalledWith(1, xxxxx) <-- check parameter for first dynamoDB call
    expect(spies[1]).toHaveBeenCalledWith(yyyyy) <-- check parameter for first lambda call
    expect(spies[0]).toHaveBeenNthCalledWith(2, zzzzz) <-- check second parameter for second dynamoDB call
 */
export declare class MockChain {
    private unnamedSpies;
    private namedSpies;
    /**
     * @returns Array of {jest.SpyInstance}
     */
    getSpies(): () => Spies;
    /**
     * @returns {Spies}
     */
    getNamedSpies(): Spies;
    spies(): Spies;
    /**
       * Adding a behavior to mock chain and stack spyInstances
       * @deprecated
       * @param func target mock function of AWS-SDK
       * @param result mock will return this value
       * @param targetSpyIndex adding mock behavior to existing spies
       *  example: when you want to mock dynamodb GetItem command twice, you will chain behavior by target spy
       *      const chain = new MockChain()
              chain
                  .add(V3.mockDynamo.getItem, 1) <-- for the first call
                  .add(V3.mockDynamo.getItem, 2, 0) <-- for the second call
          * @returns {MockChain}
          */
    add(func: MockFunctionReturnSpy, result: any, targetSpyIndex?: number): MockChain;
    /**
       * Add a mock with name
       * @deprecated
       * chain the mock to which is indexed with name
       * @param name index of spy. If the name exists in the spies stack, the mock object is chained to the existing key.
       * @param func target mock function of AWS-SDK
       * @param result mock will return this value
       *  example: when you want to mock dynamodb GetItem command twice, you will chain behavior by target spy
       *      const chain = new MockChain()
          chain
              .add('getItem', V3.mockDynamo.getItem, 1) <-- for the first call
              .add('getItem', V3.mockDynamo.getItem, 2) <-- for the second call
          * @returns {MockChain}
          */
    addWithName(name: string, func: MockFunctionReturnSpy, result: any): MockChain;
    /**
     * append a mock
     * chain the mock to which is indexed with name
     * @param name index of spy. If the name exists in the spies stack, the mock object is chained to the existing key.
     * @param func target mock function of AWS-SDK
     * @param result mock will return this value
     *  example: when you want to mock dynamodb GetItem command twice, you will chain behavior by target spy
     *      const chain = new MockChain()
     * chain
     *  .add('getItem', V3.mockDynamo.getItem, 1) <-- for the first call
     *  .add('getItem', V3.mockDynamo.getItem, 2) <-- for the second call
     * @returns {MockChain}
     */
    append(name: string, func: MockFunctionReturnSpy, result: any): MockChain;
}
